/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "cmdinvert.h"
#include "graleshellinstance.h"
#include "graleshellgeneticalgorithm.h"
#include "graleshellgaparams.h"
#include <shellp/iosystem.h>
#include <grale/lensinversiongenomebase.h>
#include <grale/gridlensinversiongafactoryparams.h>
#include <grale/gridlensinversiongafactoryparamsnew.h>
#include <grale/arealensinversiongafactoryparams.h>
#include <grale/treelensinversiongafactoryparams.h>
#include <grale/imagesdata.h>
#include <grale/constants.h>
#include <grale/grid.h>
#include <grale/galensmodule.h>
#include <grale/gravitationallens.h>
#include <serut/dummyserializer.h>
#include <serut/memoryserializer.h>
#include <enut/ipv4address.h>
#include <stdlib.h>
#include <stdio.h>

#include <iostream>

#include "debugnew.h"

using namespace shellp;

double GetMassEstimate(double D_d, const std::list<grale::ImagesDataExtended *> &images)
{
	double massestimate = 0;
	int count = 0;

	std::list<grale::ImagesDataExtended *>::const_iterator it;

	for (it = images.begin() ; it != images.end() ; it++)
	{
		if ((*it)->getNumberOfImages() > 1) // this avoids taking null-space or critical lines images data into account
		{
			count++;

			grale::ImagesDataExtended *imgdata = (*it);

			grale::Vector2D<double> topright = imgdata->getTopRightCorner();
			grale::Vector2D<double> bottomleft = imgdata->getBottomLeftCorner();
			grale::Vector2D<double> diff = topright-bottomleft;
			double sizefact = MIN(diff.getX(),diff.getY());

			diff /= sizefact;
			diff /= 2.0;

			double M = diff.getLengthSquared()*((D_d*sizefact*sizefact*SPEED_C*SPEED_C)/(4.0*CONST_G))*(imgdata->getD_s()/imgdata->getD_ds());

			massestimate += M;
		}
	}

	if (count == 0)
		count = 1;

	massestimate /= (double)count;

	return massestimate;
}

CmdInvertBase::CmdInvertBase(const std::string &cmdname) : ShellCommand(cmdname)
{
}

CmdInvertBase::~CmdInvertBase()
{
}

bool CmdInvertBase::ProcessSolution(mogal::GeneticAlgorithm &ga)
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	GraleShellGAParams *params = gsinst->GetGAParams();

	if (params->SaveAll())
	{
		char buf[256];
		int num = 1;
		std::list<mogal::Genome *> bestgenomes;
		std::list<mogal::Genome *>::const_iterator it;
		std::string prefix = params->GetSavePrefix();
		IOSystem *iosys = gsinst->getIOSystem();

		ga.getBestGenomes(bestgenomes);

		for (it = bestgenomes.begin() ; it != bestgenomes.end() ; it++,num++)
		{
			sprintf(buf, "_%02d.lensdata", num);
			std::string fname = prefix+std::string(buf);

			const grale::LensInversionGenomeBase *genome = (const grale::LensInversionGenomeBase *)(*it);
			double totalmass;
			std::string errstr;
			
			grale::GravitationalLens *lens = genome->createLens(&totalmass,errstr);
		
			if (!lens)
				iosys->writeOutputLine(std::string("WARNING: Couldn't create lens to save final solution: ") + errstr);
			else
			{
				if (!lens->save(fname))
					iosys->writeOutputLine(std::string("WARNING: Couldn't save final solution as ") + fname + std::string(": ") + lens->getErrorString());
			}
			delete lens;
		}
	}

	const grale::LensInversionGenomeBase *genome = (const grale::LensInversionGenomeBase *)ga.selectPreferredGenome();

	if (genome == 0)
	{
		setErrorString(ga.getErrorString());
		return false;
	}

	{
		char buf[1024];
		sprintf(buf, "Selected genome has fitness: %s", genome->getFitnessDescription().c_str());
		gsinst->getIOSystem()->writeOutputLine(buf);
	}

	std::string errstr;
	double totalmass;
	
	// Note: this lens is created in a shared object and will be invalid once it is unloaded.
	grale::GravitationalLens *lens = genome->createLens(&totalmass,errstr);
	if (lens == 0)
	{
		setErrorString(std::string("Unable to create lens from genome: ")+ errstr);
		return false;
	}
	
	//if (totalmass > 0)
	{
		char buf[1024];
		sprintf(buf,"Total retrieved mass is %g", (double)(totalmass/gsinst->GetMassUnit()));
		gsinst->getIOSystem()->writeOutputLine(buf);
	}

	// We'll create a local copy of the lens by serializing it
	serut::DummySerializer counter;
	uint8_t *buf;
	
	lens->write(counter);
	buf = new uint8_t [counter.getBytesWritten()];

	serut::MemorySerializer mser(buf, counter.getBytesWritten(), buf, counter.getBytesWritten());
	lens->write(mser);

	// at this point, we don't need the lens anymore
	delete lens;
	
	grale::GravitationalLens *newlens = 0;
	if (!grale::GravitationalLens::read(mser, &newlens, errstr))
	{
		setErrorString(std::string("Unable to create local copy of the lens from genome: ") + errstr);
		delete [] buf;
		return false;
	}

	delete [] buf;
	gsinst->SetLens(newlens);

	return true;
}
	
bool CmdInvertBase::GetModuleDirectory(std::string &basedir)
{
	// TODO: find a good solution for the module path

	char *path = getenv("GRALE_MODULEPATH");
	if (path == 0)
	{
		setErrorString("The environment variable GRALE_MODULEPATH is not set");
		return false;
	}

	basedir = std::string(path);
	return true;
}

mogal::GeneticAlgorithm *CmdInvertBase::GetGeneticAlgorithm()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	GraleShellGAParams *params = gsinst->GetGAParams();

	if (params->SaveIntermediate())
		return new GraleShellGeneticAlgorithm(params->GetIntermediatePrefix());
	return new GraleShellGeneticAlgorithm();
}

CmdInvertGridBase::CmdInvertGridBase(const std::string &cmdname) : CmdInvertBase(cmdname)
{
}

CmdInvertGridBase::~CmdInvertGridBase()
{
	delete invtypearg;
	delete Ddarg;
	delete plumwidtharg;
	delete popsizearg;
	delete maxgenarg;
	delete adjustweights;
	delete basisarg;
	delete m_pAllowNegativeArg;
	delete m_pUseBaseLensArg;
	delete m_pMassScaleArg;
	delete m_pSearchSheetArg;
	delete m_pZArg;
}

void CmdInvertGridBase::AddDefaultArguments()
{
	Ddarg = new ShellCmdRealArg("D_d");
	Ddarg->setUnitType(UNIT_DISTANCE);
	Ddarg->setDescription("Angular diameter distance between lens and observer.");
	addArgument(Ddarg);
	m_pZArg = new ShellCmdRealArg("z_lens");
	m_pZArg->setDescription("The redshift of the lens (should only matter if time-delay data is used).");
	addArgument(m_pZArg);
	invtypearg = new ShellCmdStringArg("algorithm");
	invtypearg->setDescription("Specifies the algorithm which should be used for the lens inversion procedure.");
	addArgument(invtypearg);
	popsizearg = new ShellCmdIntArg("populationsize");
	popsizearg->setDefault("500");
	popsizearg->setDescription("Number of genomes per generation.");
	popsizearg->setMin(10);
	addArgument(popsizearg);
	maxgenarg = new ShellCmdIntArg("maxgenerations");
	maxgenarg->setDefault("15000");
	maxgenarg->setDescription("Maximum number of generations after which the algorithm should terminate.");
	maxgenarg->setMin(1);
	addArgument(maxgenarg);
	plumwidtharg = new ShellCmdRealArg("widthfactor");
	plumwidtharg->setDefault("1.7");
	plumwidtharg->setDescription("Factor by which the size of a grid square is multiplied to obtain the width of the corresponding Plummer basis function. Not used if square shaped basis functions are used.");
	addArgument(plumwidtharg);
	adjustweights = new ShellCmdBoolArg("adjustweights");
	adjustweights->setDefault("no");
	adjustweights->setDescription("Adjust the weights to make the genome reflect the heights of the basis functions instead of their masses.");
	addArgument(adjustweights);
	basisarg = new ShellCmdStringArg("basisfunction");
	basisarg->setDefault("plummer");
	basisarg->setDescription("Select the basis function to be used. Should be either 'plummer', 'square' or 'gaussian'.");
	addArgument(basisarg);
	m_pAllowNegativeArg = new ShellCmdBoolArg("allowneg");
	m_pAllowNegativeArg->setDefault("no");
	m_pAllowNegativeArg->setDescription("Flag indicating if negative densities should be allowed.");
	addArgument(m_pAllowNegativeArg);
	m_pUseBaseLensArg = new ShellCmdBoolArg("usebaselens");
	m_pUseBaseLensArg->setDefault("no");
	m_pUseBaseLensArg->setDescription("If set, the current lens will be used as the base lens. The lens returned by the inversion procedure will be a correction on this model.");
	addArgument(m_pUseBaseLensArg);
	m_pMassScaleArg = new ShellCmdRealArg("massscale");
	m_pMassScaleArg->setDefault("-1");
	m_pMassScaleArg->setUnitType(UNIT_MASS);
	m_pMassScaleArg->setDescription("Typical mass scale of the lens. A negative value means that a mass scale will be estimated from the positions and distances of the images.");
	addArgument(m_pMassScaleArg);
	m_pSearchSheetArg = new ShellCmdStringArg("sheetsearch");
	m_pSearchSheetArg->setDefault("none");
	m_pSearchSheetArg->setDescription("If set to 'none', the algorithm will not try to search for an appropriate mass-sheet. Other valid choices are 'genome' and 'loop', specifying if a value "
			                  "in the genome represents a mass sheet or an appropriate sheet value will be sought automatically in an inner loop.");
	addArgument(m_pSearchSheetArg);
}

mogal::GAFactoryParams *CmdInvertGridBase::GetFactoryParameters()
{
	std::list<grale::ImagesDataExtended *> imgslist;

	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	gsinst->GetImagesDataList(imgslist);

	if (imgslist.empty())
	{
		setErrorString("No images data present in list");
		return 0;
	}

	grale::Grid *grid = gsinst->GetGrid();
	if (grid == 0)
	{
		setErrorString("No grid available");
		return 0;
	}
	
	std::list<grale::GridSquare> gridsquares;
	std::list<grale::GridSquare> scaledgridsquares;
	grid->getSquares(gridsquares);

	if (gridsquares.size() == 0)
	{
		setErrorString("Grid doesn't contain any squares");
		return 0;
	}
	
	grale::GridLensInversionGAFactoryParams::BasisFunctionType b;
	
	if (basisarg->getValue() == std::string("plummer"))
		b = grale::GridLensInversionGAFactoryParams::PlummerBasis;
	else if (basisarg->getValue() == std::string("square"))
		b = grale::GridLensInversionGAFactoryParams::SquareBasis;
	else if (basisarg->getValue() == std::string("gaussian"))
		b = grale::GridLensInversionGAFactoryParams::GaussBasis;
	else
	{
		setErrorString("Invalid basis function was specified");
		return 0;
	}
	
	grale::GravitationalLens *pLens = 0;

	if (m_pUseBaseLensArg->getValue())
	{
		pLens = gsinst->GetLens();
		if (pLens == 0)
		{
			setErrorString("No lens has been created yet");
			return false;
		}
	}
	
	std::list<grale::GridSquare>::const_iterator it;
	double factor = 1.0;
	
	if (b == grale::GridLensInversionGAFactoryParams::PlummerBasis) // TODO: factor for gaussians?
		factor = plumwidtharg->getValue();

	// TODO: for testing
	if (b == grale::GridLensInversionGAFactoryParams::GaussBasis)
		factor = 1.0;

	std::cout << "Factor is: " << factor << std::endl;

	for (it = gridsquares.begin() ; it != gridsquares.end() ; it++)
		scaledgridsquares.push_back(grale::GridSquare((*it).getCenter(),(*it).getSize()*factor));

	char buf[256];
	double massEstimate = m_pMassScaleArg->getValue();

	if (massEstimate < 0)
	{
		massEstimate = GetMassEstimate(Ddarg->getValue(), imgslist);
		sprintf(buf,"Using calculated mass estimate %g", (double)(massEstimate/gsinst->GetMassUnit()));
	}
	else
		sprintf(buf,"Using specified mass estimate %g", (double)(massEstimate/gsinst->GetMassUnit()));
	
	gsinst->getIOSystem()->writeOutputLine(buf);

	grale::GridLensInversionGAFactoryParams::MassSheetSearchType sheetSearchType = grale::GridLensInversionGAFactoryParams::None;

	if (m_pSearchSheetArg->getValue() == std::string("none"))
		sheetSearchType = grale::GridLensInversionGAFactoryParams::None;
	else if (m_pSearchSheetArg->getValue() == std::string("genome"))
		sheetSearchType = grale::GridLensInversionGAFactoryParams::Genome;
	else if (m_pSearchSheetArg->getValue() == std::string("loop"))
		sheetSearchType = grale::GridLensInversionGAFactoryParams::Loop;
	else
	{
		setErrorString("Invalid sheet search type was specified (valid choices are 'none', 'genome' and 'loop')");
		return false;
	}

	return new grale::GridLensInversionGAFactoryParams(maxgenarg->getValue(), imgslist, scaledgridsquares,
			                                   Ddarg->getValue(), m_pZArg->getValue(), massEstimate, false, 
			                                   adjustweights->getValue(), b, m_pAllowNegativeArg->getValue(), 
							   pLens, sheetSearchType);
}

bool CmdInvertGridBase::GetModuleName(std::string &modulename)
{
	modulename = std::string("libgalens_") + invtypearg->getValue() + std::string(".so");
	return true;
}

CmdInvertGridLocal::CmdInvertGridLocal(const std::string &cmdname) : CmdInvertGridBase(cmdname)
{
	AddDefaultArguments();
	setDescription("Use a genetic algorithm which is executed locally to invert a lens. The images in the images data list are used and Plummer basis functions are placed corresponding to the current grid layout.");
}

CmdInvertGridLocal::~CmdInvertGridLocal()
{
}

bool CmdInvertGridLocal::execute()
{
	std::string basedir;
	std::string modulename;

	if (!GetModuleDirectory(basedir))
		return false;
	if (!GetModuleName(modulename))
		return false;
	
	mogal::GAFactoryParams *params = GetFactoryParameters();
	if (params == 0)
		return false;

	mogal::GeneticAlgorithm *ga = GetGeneticAlgorithm();
	if (ga == 0)
	{
		delete params;
		return false;
	}

	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!ga->run(modulename, GetPopulationSize(), basedir, params, gsinst->GetGAParams()))
	{
		delete params;
		setErrorString(std::string("Error running genetic algorithm: ") + ga->getErrorString());
		delete ga;
		return false;
	}
	delete params;

	if (!ProcessSolution(*ga))
	{
		delete ga;
		return false;
	}
	delete ga;
	
	return true;
}

CmdInvertGridRemote::CmdInvertGridRemote(const std::string &cmdname) : CmdInvertGridBase(cmdname)
{
	ipstrarg = new ShellCmdStringArg("IPaddress");
	ipstrarg->setDescription("IP address of the genetic algorithm server.");
	addArgument(ipstrarg);
	portarg = new ShellCmdIntArg("port");
	portarg->setDescription("Port number of the genetic algorithm server.");
	portarg->setMin(1);
	portarg->setMax(65535);
	addArgument(portarg);

	AddDefaultArguments();
	
	setDescription("Use a genetic algorithm to invert a lens. A genetic algorithm server is used, making it "
		       "possible to distribute the fitness calculations. The images in the images data list are "
		       "used and Plummer basis functions are placed corresponding to the current grid layout.");
}

CmdInvertGridRemote::~CmdInvertGridRemote()
{
	delete ipstrarg;
	delete portarg;
}

bool CmdInvertGridRemote::execute()
{
	nut::IPv4Address address;

	if (!address.setAddress(ipstrarg->getValue()))
	{
		setErrorString("An invalid IP address was entered");
		return false;
	}
	
	uint16_t port = (uint16_t)(portarg->getValue());

	std::string basedir;
	std::string modulename;
	
	if (!GetModuleDirectory(basedir))
		return false;
	if (!GetModuleName(modulename))
		return false;
	
	mogal::GAFactoryParams *params = GetFactoryParameters();
	if (params == 0)
		return false;
	
	mogal::GeneticAlgorithm *ga = GetGeneticAlgorithm();
	if (ga == 0)
	{
		delete params;
		return false;
	}
	
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!ga->run(address, port, modulename, GetPopulationSize(), basedir, params, gsinst->GetGAParams()))
	{
		delete params;
		setErrorString(std::string("Error running genetic algorithm: ") + ga->getErrorString());
		delete ga;
		return false;
	}
	delete params;

	if (!ProcessSolution(*ga))
	{
		delete ga;
		return false;
	}
	delete ga;
	
	return true;
}

CmdInvertGAParamsShow::CmdInvertGAParamsShow(const std::string &cmdname) : ShellCommand(cmdname)
{
	setDescription("Displays the current genetic algorithm parameters.");
}

CmdInvertGAParamsShow::~CmdInvertGAParamsShow()
{
}

bool CmdInvertGAParamsShow::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	IOSystem *iosys = gsinst->getIOSystem();
	GraleShellGAParams *params = gsinst->GetGAParams();
	
	char buf[1024];

	iosys->writeOutputLine("");

	sprintf(buf, "  Selection pressure (beta):     %g", (double)params->getBeta());
	iosys->writeOutputLine(buf);

	sprintf(buf, "  Use elitism:                   %s", (params->useElitism()?"yes":"no"));
	iosys->writeOutputLine(buf);

	sprintf(buf, "  Always include best genome:    %s", (params->alwaysIncludeBestGenome()?"yes":"no"));
	iosys->writeOutputLine(buf);

	sprintf(buf, "  Cross-over rate:               %g", (double)params->getCrossOverRate());
	iosys->writeOutputLine(buf);
	
	sprintf(buf, "  Save intermediate solutions:   %s", (params->SaveIntermediate()?"yes":"no"));
	iosys->writeOutputLine(buf);

	sprintf(buf, "  Intermediate save file prefix: %s", params->GetIntermediatePrefix().c_str());
	iosys->writeOutputLine(buf);
	
	sprintf(buf, "  Save all final solutions:      %s", (params->SaveAll()?"yes":"no"));
	iosys->writeOutputLine(buf);

	sprintf(buf, "  Final save file prefix:        %s", params->GetSavePrefix().c_str());
	iosys->writeOutputLine(buf);
		
	iosys->writeOutputLine("");

	return true;
}

CmdInvertGAParamsSet::CmdInvertGAParamsSet(const std::string &cmdname) : ShellCommand(cmdname)
{
	betaarg = new ShellCmdRealArg("beta");
	betaarg->setMin(0.0);
	betaarg->setDefault("2.5");
	betaarg->setDescription("Sets the selection pressure beta. If the position of a genome is 'n' after "
			        "the ranking procedure (with 'n' being higher for better genomes), the probability "
				"of it being selected for reproduction or cloning is proportional to n^beta. A "
				"value of zero causes genomes to be selected at random, while higher values of "
				"beta favor fitter genomes.");
	addArgument(betaarg);
	elitarg = new ShellCmdBoolArg("elitism");
	elitarg->setDefault("yes");
	elitarg->setDescription("If elitism is used, the best genome of a generation is copied to the next "
			        "generation. Afterwards, it is still subjected to the mutation procedure.");
	addArgument(elitarg);
	bestarg = new ShellCmdBoolArg("keepbest");
	bestarg->setDefault("yes");
	bestarg->setDescription("If set to true, the best genome of a generation is copied to the next "
			        "generation and is not subjected to mutations afterwards.");
	addArgument(bestarg);
	crossarg = new ShellCmdRealArg("crossoverrate");
	crossarg->setDefault("0.9");
	crossarg->setMin(0.0);
	crossarg->setMax(1.0);
	crossarg->setDescription("Sets the fraction of genomes in a generation which are obtained by "
			         "reproduction. The other genomes are created by cloning.");
	addArgument(crossarg);

	saveintarg = new ShellCmdBoolArg("saveintermediate");
	saveintarg->setDescription("Flag indicating if intermediate solutions should be saved.");
	saveintarg->setDefault("no");
	addArgument(saveintarg);

	prefarg = new ShellCmdStringArg("saveintprefix");
	prefarg->setDescription("If intermediate solutions are saved, this filename prefix is used.");
	prefarg->setDefault("intlens_");
	addArgument(prefarg);
	
	saveallarg = new ShellCmdBoolArg("saveall");
	saveallarg->setDescription("Flag indicating if all final solutions should be saved. In the case of a multi-objective genetic algorithm, this allows you to save all retrieved pareto-optimal solutions.");
	saveallarg->setDefault("no");
	addArgument(saveallarg);

	saveprefarg = new ShellCmdStringArg("saveallprefix");
	saveprefarg->setDescription("If all final solutions are saved, this filename prefix is used.");
	saveprefarg->setDefault("savedlens_");
	addArgument(saveprefarg);

	setDescription("Sets the parameters to be used in the genetic algorithm for lens inversion.");
}

CmdInvertGAParamsSet::~CmdInvertGAParamsSet()
{
	delete betaarg;
	delete elitarg;
	delete bestarg;
	delete crossarg;
	delete saveintarg;
	delete prefarg;
	delete saveallarg;
	delete saveprefarg;
}

bool CmdInvertGAParamsSet::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	GraleShellGAParams *gaparams = new GraleShellGAParams(betaarg->getValue(), elitarg->getValue(), bestarg->getValue(), crossarg->getValue(), saveintarg->getValue(), prefarg->getValue(), saveallarg->getValue(), saveprefarg->getValue());
	
	gsinst->SetGAParams(gaparams);
	
	return true;
}

CmdInvertAreaBase::CmdInvertAreaBase(const std::string &cmdname) : CmdInvertBase(cmdname)
{
}

CmdInvertAreaBase::~CmdInvertAreaBase()
{
	delete invtypearg;
	delete sizearg;
	delete xarg;
	delete yarg;
	delete Ddarg;
	delete popsizearg;
	delete maxgenarg;
	delete filearg;
}

void CmdInvertAreaBase::AddDefaultArguments()
{
	Ddarg = new ShellCmdRealArg("D_d");
	Ddarg->setUnitType(UNIT_DISTANCE);
	Ddarg->setDescription("Angular diameter distance between lens and observer.");
	addArgument(Ddarg);
	sizearg = new ShellCmdRealArg("size");
	sizearg->setUnitType(UNIT_ANGULAR);
	sizearg->setDescription("Size of the area in which the lens reconstruction should take place.");
	addArgument(sizearg);
	xarg = new ShellCmdRealArg("x");
	xarg->setUnitType(UNIT_ANGULAR);
	xarg->setDescription("X coordinate of the center of the area in which the lens reconstruction should take place.");
	xarg->setDefault("0");
	addArgument(xarg);
	yarg = new ShellCmdRealArg("y");
	yarg->setUnitType(UNIT_ANGULAR);
	yarg->setDescription("Y coordinate of the center of the area in which the lens reconstruction should take place.");
	yarg->setDefault("0");
	addArgument(yarg);
	invtypearg = new ShellCmdStringArg("algorithm");
	invtypearg->setDefault("nsiefit");
	invtypearg->setDescription("Specifies the algorithm which should be used for the lens inversion procedure.");
	addArgument(invtypearg);
	popsizearg = new ShellCmdIntArg("populationsize");
	popsizearg->setDefault("500");
	popsizearg->setDescription("Number of genomes per generation.");
	popsizearg->setMin(10);
	addArgument(popsizearg);
	maxgenarg = new ShellCmdIntArg("maxgenerations");
	maxgenarg->setDefault("15000");
	maxgenarg->setDescription("Maximum number of generations after which the algorithm should terminate.");
	maxgenarg->setMin(1);
	addArgument(maxgenarg);
	filearg = new ShellCmdStringArg("filename");
	filearg->setDefault("*");
	filearg->setDescription("If specified, extra parameters for the algorithm will be read from this file.");
	addArgument(filearg);
}

bool CmdInvertAreaBase::GetModuleName(std::string &modulename)
{
	modulename = std::string("libgalens_") + invtypearg->getValue() + std::string(".so");
	return true;
}

mogal::GAFactoryParams *CmdInvertAreaBase::GetFactoryParameters(mogal::GAFactory &factory)
{
	std::list<grale::ImagesDataExtended *> imgslist;

	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	gsinst->GetImagesDataList(imgslist);

	if (imgslist.empty())
	{
		setErrorString("No images data present in list");
		return 0;
	}
	
	grale::LensInversionGAFactoryParams *p = (grale::LensInversionGAFactoryParams *)factory.createParamsInstance();
	if (p->getInversionType() != grale::LensInversionGAFactoryParams::AreaInversion)
	{
		setErrorString("The specified algorithm is not an area based one");
		delete p;
		return 0;
	}

	grale::AreaLensInversionGAFactoryParams *params = (grale::AreaLensInversionGAFactoryParams *)p;
	params->init(maxgenarg->getValue(), imgslist, Ddarg->getValue(), grale::Vector2D<double>(xarg->getValue(), yarg->getValue()), sizearg->getValue(), false);

	if (filearg->getValue() != std::string("*"))
	{
		if (!params->loadExtraParameters(filearg->getValue()))
		{
			setErrorString(std::string("Unable to load extra parameters from file:") + params->getErrorString());
			delete params;
			return 0;
		}
	}
	return params;
}

CmdInvertAreaLocal::CmdInvertAreaLocal(const std::string &cmdname) : CmdInvertAreaBase(cmdname)
{
	AddDefaultArguments();
	// TODO fix description
}

CmdInvertAreaLocal::~CmdInvertAreaLocal()
{
}

bool CmdInvertAreaLocal::execute()
{
	std::string basedir;
	std::string modulename;

	if (!GetModuleDirectory(basedir))
		return false;
	if (!GetModuleName(modulename))
		return false;

	grale::GALensModule gamodule;

	if (!gamodule.open(basedir, modulename))
	{
		setErrorString(std::string("Couldn't load specified algorithm: ") + gamodule.getErrorString());
		return false;
	}
	
	mogal::GAFactory *factory = gamodule.createFactoryInstance();
	if (factory == 0)
	{
		setErrorString("Unable to allocate a new genome factory for the specified algorithm");
		return false;
	}
	
	mogal::GAFactoryParams *params = GetFactoryParameters(*factory);
	if (params == 0)
	{
		delete factory;
		return false;
	}

	if (!factory->init(params))
	{
		setErrorString(std::string("Couldn't initialize the specified algorithm: ") + factory->getErrorString());
		delete params;
		delete factory;
		return false;
	}

	// don't need these parameters anymore
	delete params;
	
	mogal::GeneticAlgorithm *ga = GetGeneticAlgorithm();
	if (ga == 0)
	{
		delete params;
		return false;
	}
	
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!ga->run(*factory, GetPopulationSize(), gsinst->GetGAParams()))
	{
		setErrorString(std::string("Error running genetic algorithm: ") + ga->getErrorString());
		delete ga;
		delete factory;
		return false;
	}

	if (!ProcessSolution(*ga))
	{
		delete ga;
		delete factory;
		return false;
	}
	delete ga;
	delete factory;
	
	return true;
}

CmdInvertAreaRemote::CmdInvertAreaRemote(const std::string &cmdname) : CmdInvertAreaBase(cmdname)
{
	ipstrarg = new ShellCmdStringArg("IPaddress");
	ipstrarg->setDescription("IP address of the genetic algorithm server.");
	addArgument(ipstrarg);
	portarg = new ShellCmdIntArg("port");
	portarg->setDescription("Port number of the genetic algorithm server.");
	portarg->setMin(1);
	portarg->setMax(65535);
	addArgument(portarg);

	AddDefaultArguments();
	
	// TODO fix description
	setDescription("Use a genetic algorithm to invert a lens. A genetic algorithm server is used, making it "
		       "possible to distribute the fitness calculations. The images in the images data list are "
		       "used and Plummer basis functions are placed corresponding to the current grid layout.");
}

CmdInvertAreaRemote::~CmdInvertAreaRemote()
{
	delete ipstrarg;
	delete portarg;
}

bool CmdInvertAreaRemote::execute()
{
	nut::IPv4Address address;

	if (!address.setAddress(ipstrarg->getValue()))
	{
		setErrorString("An invalid IP address was entered");
		return false;
	}
	
	uint16_t port = (uint16_t)(portarg->getValue());

	std::string basedir;
	std::string modulename;

	if (!GetModuleDirectory(basedir))
		return false;
	if (!GetModuleName(modulename))
		return false;

	grale::GALensModule gamodule;

	if (!gamodule.open(basedir, modulename))
	{
		setErrorString(std::string("Couldn't load specified algorithm: ") + gamodule.getErrorString());
		return false;
	}
	
	mogal::GAFactory *factory = gamodule.createFactoryInstance();
	if (factory == 0)
	{
		setErrorString("Unable to allocate a new genome factory for the specified algorithm");
		return false;
	}
	
	mogal::GAFactoryParams *params = GetFactoryParameters(*factory);
	if (params == 0)
	{
		delete factory;
		return false;
	}

	if (!factory->init(params))
	{
		setErrorString(std::string("Couldn't initialize the specified algorithm: ") + factory->getErrorString());
		delete params;
		delete factory;
		return false;
	}

	// don't need these parameters anymore
	delete params;
	
	mogal::GeneticAlgorithm *ga = GetGeneticAlgorithm();
	if (ga == 0)
	{
		delete params;
		return false;
	}
	
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!ga->run(address, port, modulename, GetPopulationSize(), *factory, gsinst->GetGAParams()))
	{
		setErrorString(std::string("Error running genetic algorithm: ") + ga->getErrorString());
		delete ga;
		delete factory;
		return false;
	}

	if (!ProcessSolution(*ga))
	{
		delete ga;
		delete factory;
		return false;
	}
	delete ga;
	delete factory;

	return true;
}

CmdInvertTreeBase::CmdInvertTreeBase(const std::string &cmdName) : CmdInvertBase(cmdName)
{
}

CmdInvertTreeBase::~CmdInvertTreeBase()
{
	delete m_pInversionModuleArg;
	delete m_pSizeArg;
	delete m_pCenterXArg;
	delete m_pCenterYArg;
	delete m_pDdArg;
	delete m_pPopulationSizeArg;
	delete m_pMaxGenArg;
	delete m_pMaxDepthArg;
	delete m_pMaxBasisArg;
	delete m_pUseWeightsArg;
	delete m_pFirstSubdivArg;
}

// TODO: add width scale factor, basis function type
void CmdInvertTreeBase::AddDefaultArguments()
{
	m_pDdArg = new ShellCmdRealArg("D_d");
	m_pDdArg->setUnitType(UNIT_DISTANCE);
	m_pDdArg->setDescription("Angular diameter distance between lens and observer.");
	addArgument(m_pDdArg);
	m_pSizeArg = new ShellCmdRealArg("size");
	m_pSizeArg->setUnitType(UNIT_ANGULAR);
	m_pSizeArg->setDescription("Size of the area in which the lens reconstruction should take place.");
	addArgument(m_pSizeArg);
	m_pCenterXArg = new ShellCmdRealArg("x");
	m_pCenterXArg->setUnitType(UNIT_ANGULAR);
	m_pCenterXArg->setDescription("X coordinate of the center of the area in which the lens reconstruction should take place.");
	m_pCenterXArg->setDefault("0");
	addArgument(m_pCenterXArg);
	m_pCenterYArg = new ShellCmdRealArg("y");
	m_pCenterYArg->setUnitType(UNIT_ANGULAR);
	m_pCenterYArg->setDescription("Y coordinate of the center of the area in which the lens reconstruction should take place.");
	m_pCenterYArg->setDefault("0");
	addArgument(m_pCenterYArg);
	m_pInversionModuleArg = new ShellCmdStringArg("algorithm");
	m_pInversionModuleArg->setDescription("Specifies the algorithm which should be used for the lens inversion procedure.");
	addArgument(m_pInversionModuleArg);
	m_pPopulationSizeArg = new ShellCmdIntArg("populationsize");
	m_pPopulationSizeArg->setDefault("500");
	m_pPopulationSizeArg->setDescription("Number of genomes per generation.");
	m_pPopulationSizeArg->setMin(10);
	addArgument(m_pPopulationSizeArg);
	m_pFirstSubdivArg = new ShellCmdIntArg("firstsubdiv");
	m_pFirstSubdivArg->setDefault("15");
	m_pFirstSubdivArg->setDescription("The initial number of cells will equal the square of this value.");
	m_pFirstSubdivArg->setMin(2);
	addArgument(m_pFirstSubdivArg);
	m_pMaxGenArg = new ShellCmdIntArg("maxgenerations");
	m_pMaxGenArg->setDefault("15000");
	m_pMaxGenArg->setDescription("Maximum number of generations after which the algorithm should terminate.");
	m_pMaxGenArg->setMin(1);
	addArgument(m_pMaxGenArg);
	m_pMaxDepthArg = new ShellCmdIntArg("maxdepth");
	m_pMaxDepthArg->setDefault("4");
	m_pMaxDepthArg->setMin(1);
	m_pMaxDepthArg->setDescription("Maximum allowed depth of the grid tree.");
	addArgument(m_pMaxDepthArg);
	m_pMaxBasisArg = new ShellCmdIntArg("maxbasis");
	m_pMaxBasisArg->setDefault("1500");
	m_pMaxBasisArg->setMin(10);
	m_pMaxBasisArg->setDescription("Maximum allowed number of basis functions.");
	addArgument(m_pMaxBasisArg);
	m_pUseWeightsArg = new ShellCmdBoolArg("useweights");
	m_pUseWeightsArg->setDefault("no");
	m_pUseWeightsArg->setDescription("If set, this flag will cause the algorithm to use the densities of basis functions as "
			                 "parameter, instead of the masses.");
	addArgument(m_pUseWeightsArg);
}

bool CmdInvertTreeBase::GetModuleName(std::string &moduleName)
{
	moduleName = std::string("libgalens_") + m_pInversionModuleArg->getValue() + std::string(".so");
	return true;
}

mogal::GAFactoryParams *CmdInvertTreeBase::GetFactoryParameters()
{
	std::list<grale::ImagesDataExtended *> imagesList;

	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	pGSInst->GetImagesDataList(imagesList);

	if (imagesList.empty())
	{
		setErrorString("No images data present in list");
		return 0;
	}

	double massScale = GetMassEstimate(m_pDdArg->getValue(), imagesList);
	
	grale::TreeLensInversionGAFactoryParams *pParams = new grale::TreeLensInversionGAFactoryParams(
									m_pMaxGenArg->getValue(), 
									imagesList, 
									grale::Vector2D<double>(m_pCenterXArg->getValue(), m_pCenterYArg->getValue()),
									m_pSizeArg->getValue(),
									m_pMaxDepthArg->getValue(),
									m_pMaxBasisArg->getValue(),
									m_pDdArg->getValue(), 
									massScale,
									false,
									1.7, // TODO: make this configurable
									m_pUseWeightsArg->getValue(),
									m_pFirstSubdivArg->getValue());
	return pParams;
}

CmdInvertTreeLocal::CmdInvertTreeLocal(const std::string &cmdname) : CmdInvertTreeBase(cmdname)
{
	AddDefaultArguments();
}

CmdInvertTreeLocal::~CmdInvertTreeLocal()
{
}

bool CmdInvertTreeLocal::execute()
{
	std::string basedir;
	std::string modulename;

	if (!GetModuleDirectory(basedir))
		return false;
	if (!GetModuleName(modulename))
		return false;

	grale::GALensModule gamodule;

	if (!gamodule.open(basedir, modulename))
	{
		setErrorString(std::string("Couldn't load specified algorithm: ") + gamodule.getErrorString());
		return false;
	}
	
	mogal::GAFactory *factory = gamodule.createFactoryInstance();
	if (factory == 0)
	{
		setErrorString("Unable to allocate a new genome factory for the specified algorithm");
		return false;
	}
	
	mogal::GAFactoryParams *params = GetFactoryParameters();
	if (params == 0)
	{
		delete factory;
		return false;
	}

	if (!factory->init(params))
	{
		setErrorString(std::string("Couldn't initialize the specified algorithm: ") + factory->getErrorString());
		delete params;
		delete factory;
		return false;
	}

	// don't need these parameters anymore
	delete params;
	
	mogal::GeneticAlgorithm *ga = GetGeneticAlgorithm();
	if (ga == 0)
	{
		delete params;
		return false;
	}
	
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!ga->run(*factory, GetPopulationSize(), gsinst->GetGAParams()))
	{
		setErrorString(std::string("Error running genetic algorithm: ") + ga->getErrorString());
		delete ga;
		delete factory;
		return false;
	}

	if (!ProcessSolution(*ga))
	{
		delete ga;
		delete factory;
		return false;
	}
	delete ga;
	delete factory;
	
	return true;
}

CmdInvertTreeRemote::CmdInvertTreeRemote(const std::string &cmdName) : CmdInvertTreeBase(cmdName)
{
	m_pIPArg = new ShellCmdStringArg("IPaddress");
	m_pIPArg->setDescription("IP address of the genetic algorithm server.");
	addArgument(m_pIPArg);
	m_pPortArg = new ShellCmdIntArg("port");
	m_pPortArg->setDescription("Port number of the genetic algorithm server.");
	m_pPortArg->setMin(1);
	m_pPortArg->setMax(65535);
	addArgument(m_pPortArg);

	AddDefaultArguments();
	
	// TODO fix description
	setDescription("Use a genetic algorithm to invert a lens. A genetic algorithm server is used, making it "
		       "possible to distribute the fitness calculations. The images in the images data list are "
		       "used and Plummer basis functions are placed corresponding to the current grid layout.");
}

CmdInvertTreeRemote::~CmdInvertTreeRemote()
{
	delete m_pPortArg;
	delete m_pIPArg;
}

bool CmdInvertTreeRemote::execute()
{
	nut::IPv4Address address;

	if (!address.setAddress(m_pIPArg->getValue()))
	{
		setErrorString("An invalid IP address was entered");
		return false;
	}
	
	uint16_t port = (uint16_t)(m_pPortArg->getValue());

	std::string basedir;
	std::string modulename;

	if (!GetModuleDirectory(basedir))
		return false;
	if (!GetModuleName(modulename))
		return false;

	grale::GALensModule gamodule;

	if (!gamodule.open(basedir, modulename))
	{
		setErrorString(std::string("Couldn't load specified algorithm: ") + gamodule.getErrorString());
		return false;
	}
	
	mogal::GAFactory *factory = gamodule.createFactoryInstance();
	if (factory == 0)
	{
		setErrorString("Unable to allocate a new genome factory for the specified algorithm");
		return false;
	}
	
	mogal::GAFactoryParams *params = GetFactoryParameters();
	if (params == 0)
	{
		delete factory;
		return false;
	}

	if (!factory->init(params))
	{
		setErrorString(std::string("Couldn't initialize the specified algorithm: ") + factory->getErrorString());
		delete params;
		delete factory;
		return false;
	}

	// don't need these parameters anymore
	delete params;
	
	mogal::GeneticAlgorithm *ga = GetGeneticAlgorithm();
	if (ga == 0)
	{
		delete params;
		return false;
	}
	
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!ga->run(address, port, modulename, GetPopulationSize(), *factory, gsinst->GetGAParams()))
	{
		setErrorString(std::string("Error running genetic algorithm: ") + ga->getErrorString());
		delete ga;
		delete factory;
		return false;
	}

	if (!ProcessSolution(*ga))
	{
		delete ga;
		delete factory;
		return false;
	}
	delete ga;
	delete factory;

	return true;
}

CmdInvertGridNewBase::CmdInvertGridNewBase(const std::string &cmdname) : CmdInvertBase(cmdname)
{
}

CmdInvertGridNewBase::~CmdInvertGridNewBase()
{
	delete invtypearg;
	delete Ddarg;
	delete plumwidtharg;
	delete popsizearg;
	delete maxgenarg;
	delete basisarg;
	delete m_pAllowNegativeArg;
	delete m_pUseBaseLensArg;
	delete m_pMassScaleArg;
	delete m_pSearchSheetArg;
	delete m_pZArg;
	delete m_pShowEvolutionArg;
	delete m_pSaveFirstGenomeArg;
}

void CmdInvertGridNewBase::AddDefaultArguments()
{
	Ddarg = new ShellCmdRealArg("D_d");
	Ddarg->setUnitType(UNIT_DISTANCE);
	Ddarg->setDescription("Angular diameter distance between lens and observer.");
	addArgument(Ddarg);
	m_pZArg = new ShellCmdRealArg("z_lens");
	m_pZArg->setDescription("The redshift of the lens (should only matter if time-delay data is used).");
	addArgument(m_pZArg);
	invtypearg = new ShellCmdStringArg("algorithm");
	invtypearg->setDescription("Specifies the algorithm which should be used for the lens inversion procedure.");
	addArgument(invtypearg);
	popsizearg = new ShellCmdIntArg("populationsize");
	popsizearg->setDefault("500");
	popsizearg->setDescription("Number of genomes per generation.");
	popsizearg->setMin(10);
	addArgument(popsizearg);
	maxgenarg = new ShellCmdIntArg("maxgenerations");
	maxgenarg->setDefault("15000");
	maxgenarg->setDescription("Maximum number of generations after which the algorithm should terminate.");
	maxgenarg->setMin(1);
	addArgument(maxgenarg);
	plumwidtharg = new ShellCmdRealArg("widthfactor");
	plumwidtharg->setDefault("1.7");
	plumwidtharg->setDescription("Factor by which the size of a grid square is multiplied to obtain the width of the corresponding Plummer basis function. Not used if square shaped basis functions are used.");
	addArgument(plumwidtharg);
	basisarg = new ShellCmdStringArg("basisfunction");
	basisarg->setDefault("plummer");
	basisarg->setDescription("Select the basis function to be used. Should be either 'plummer', 'square' or 'gaussian'.");
	addArgument(basisarg);
	m_pAllowNegativeArg = new ShellCmdBoolArg("allowneg");
	m_pAllowNegativeArg->setDefault("no");
	m_pAllowNegativeArg->setDescription("Flag indicating if negative densities should be allowed.");
	addArgument(m_pAllowNegativeArg);
	m_pUseBaseLensArg = new ShellCmdBoolArg("usebaselens");
	m_pUseBaseLensArg->setDefault("no");
	m_pUseBaseLensArg->setDescription("If set, the current lens will be used as the base lens. The lens returned by the inversion procedure will be a correction on this model.");
	addArgument(m_pUseBaseLensArg);
	m_pMassScaleArg = new ShellCmdRealArg("massscale");
	m_pMassScaleArg->setDefault("-1");
	m_pMassScaleArg->setUnitType(UNIT_MASS);
	m_pMassScaleArg->setDescription("Typical mass scale of the lens. A negative value means that a mass scale will be estimated from the positions and distances of the images.");
	addArgument(m_pMassScaleArg);
	m_pSearchSheetArg = new ShellCmdBoolArg("searchsheet");
	m_pSearchSheetArg->setDefault("no");
	m_pSearchSheetArg->setDescription("If set to true, the algorithm will try to search for an appropriate mass-sheet.");
	addArgument(m_pSearchSheetArg);
	m_pShowEvolutionArg = new ShellCmdBoolArg("showevolution");
	m_pShowEvolutionArg->setDefault("no");
	m_pShowEvolutionArg->setDescription("If set to true, it will cause a directory to be created for each generation, containing all the (sorted) mass distributions. "
			                    "When using a server to distribute the calculations, this will cause subdirectories to be created on the server side, so USE WITH CARE!");
	addArgument(m_pShowEvolutionArg);
	m_pSaveFirstGenomeArg = new ShellCmdBoolArg("savefirstgenome");
	m_pSaveFirstGenomeArg->setDefault("no");
	m_pSaveFirstGenomeArg->setDescription("Saves the first genome in every generation to a file in a subdirectory named 'bestgenomes'. Again, USE WITH CARE!");
	addArgument(m_pSaveFirstGenomeArg);
}

mogal::GAFactoryParams *CmdInvertGridNewBase::GetFactoryParameters()
{
	std::list<grale::ImagesDataExtended *> imgslist;

	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	gsinst->GetImagesDataList(imgslist);

	if (imgslist.empty())
	{
		setErrorString("No images data present in list");
		return 0;
	}

	grale::Grid *grid = gsinst->GetGrid();
	if (grid == 0)
	{
		setErrorString("No grid available");
		return 0;
	}
	
	std::list<grale::GridSquare> gridsquares;
	std::list<grale::GridSquare> scaledgridsquares;
	grid->getSquares(gridsquares);

	if (gridsquares.size() == 0)
	{
		setErrorString("Grid doesn't contain any squares");
		return 0;
	}
	
	grale::GridLensInversionGAFactoryParamsNew::BasisFunctionType b;
	
	if (basisarg->getValue() == std::string("plummer"))
		b = grale::GridLensInversionGAFactoryParamsNew::PlummerBasis;
	else if (basisarg->getValue() == std::string("square"))
		b = grale::GridLensInversionGAFactoryParamsNew::SquareBasis;
	else if (basisarg->getValue() == std::string("gaussian"))
		b = grale::GridLensInversionGAFactoryParamsNew::GaussBasis;
	else
	{
		setErrorString("Invalid basis function was specified");
		return 0;
	}
	
	grale::GravitationalLens *pLens = 0;

	if (m_pUseBaseLensArg->getValue())
	{
		pLens = gsinst->GetLens();
		if (pLens == 0)
		{
			setErrorString("No lens has been created yet");
			return false;
		}
	}
	
	std::list<grale::GridSquare>::const_iterator it;
	double factor = 1.0;
	
	if (b == grale::GridLensInversionGAFactoryParamsNew::PlummerBasis) // TODO: factor for gaussians?
		factor = plumwidtharg->getValue();

	// TODO: for testing
	if (b == grale::GridLensInversionGAFactoryParamsNew::GaussBasis)
		factor = 1.0;

	std::cout << "Factor is: " << factor << std::endl;

	for (it = gridsquares.begin() ; it != gridsquares.end() ; it++)
		scaledgridsquares.push_back(grale::GridSquare((*it).getCenter(),(*it).getSize()*factor));

	char buf[256];
	double massEstimate = m_pMassScaleArg->getValue();

	if (massEstimate < 0)
	{
		massEstimate = GetMassEstimate(Ddarg->getValue(), imgslist);
		sprintf(buf,"Using calculated mass estimate %g", (double)(massEstimate/gsinst->GetMassUnit()));
	}
	else
		sprintf(buf,"Using specified mass estimate %g", (double)(massEstimate/gsinst->GetMassUnit()));
	
	gsinst->getIOSystem()->writeOutputLine(buf);

	grale::GridLensInversionGAFactoryParamsNew::MassSheetSearchType sheetSearchType = grale::GridLensInversionGAFactoryParamsNew::None;

	if (m_pSearchSheetArg->getValue())
		sheetSearchType = grale::GridLensInversionGAFactoryParamsNew::Genome;
	
	return new grale::GridLensInversionGAFactoryParamsNew(maxgenarg->getValue(), imgslist, scaledgridsquares,
			                                   Ddarg->getValue(), m_pZArg->getValue(), massEstimate, false, 
			                                   m_pAllowNegativeArg->getValue(), pLens, b, sheetSearchType,
							   m_pShowEvolutionArg->getValue(), m_pSaveFirstGenomeArg->getValue());
}

bool CmdInvertGridNewBase::GetModuleName(std::string &modulename)
{
	modulename = std::string("libgalens_") + invtypearg->getValue() + std::string(".so");
	return true;
}

CmdInvertGridNewLocal::CmdInvertGridNewLocal(const std::string &cmdname) : CmdInvertGridNewBase(cmdname)
{
	AddDefaultArguments();
	setDescription("Use a genetic algorithm which is executed locally to invert a lens. The images in the images data list are used and Plummer basis functions are placed corresponding to the current grid layout.");
}

CmdInvertGridNewLocal::~CmdInvertGridNewLocal()
{
}

bool CmdInvertGridNewLocal::execute()
{
	std::string basedir;
	std::string modulename;

	if (!GetModuleDirectory(basedir))
		return false;
	if (!GetModuleName(modulename))
		return false;
	
	mogal::GAFactoryParams *params = GetFactoryParameters();
	if (params == 0)
		return false;

	mogal::GeneticAlgorithm *ga = GetGeneticAlgorithm();
	if (ga == 0)
	{
		delete params;
		return false;
	}

	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!ga->run(modulename, GetPopulationSize(), basedir, params, gsinst->GetGAParams()))
	{
		delete params;
		setErrorString(std::string("Error running genetic algorithm: ") + ga->getErrorString());
		delete ga;
		return false;
	}
	delete params;

	if (!ProcessSolution(*ga))
	{
		delete ga;
		return false;
	}
	delete ga;
	
	return true;
}

CmdInvertGridNewRemote::CmdInvertGridNewRemote(const std::string &cmdname) : CmdInvertGridNewBase(cmdname)
{
	ipstrarg = new ShellCmdStringArg("IPaddress");
	ipstrarg->setDescription("IP address of the genetic algorithm server.");
	addArgument(ipstrarg);
	portarg = new ShellCmdIntArg("port");
	portarg->setDescription("Port number of the genetic algorithm server.");
	portarg->setMin(1);
	portarg->setMax(65535);
	addArgument(portarg);

	AddDefaultArguments();
	
	setDescription("Use a genetic algorithm to invert a lens. A genetic algorithm server is used, making it "
		       "possible to distribute the fitness calculations. The images in the images data list are "
		       "used and Plummer basis functions are placed corresponding to the current grid layout.");
}

CmdInvertGridNewRemote::~CmdInvertGridNewRemote()
{
	delete ipstrarg;
	delete portarg;
}

bool CmdInvertGridNewRemote::execute()
{
	nut::IPv4Address address;

	if (!address.setAddress(ipstrarg->getValue()))
	{
		setErrorString("An invalid IP address was entered");
		return false;
	}
	
	uint16_t port = (uint16_t)(portarg->getValue());

	std::string basedir;
	std::string modulename;
	
	if (!GetModuleDirectory(basedir))
		return false;
	if (!GetModuleName(modulename))
		return false;
	
	mogal::GAFactoryParams *params = GetFactoryParameters();
	if (params == 0)
		return false;
	
	mogal::GeneticAlgorithm *ga = GetGeneticAlgorithm();
	if (ga == 0)
	{
		delete params;
		return false;
	}
	
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!ga->run(address, port, modulename, GetPopulationSize(), basedir, params, gsinst->GetGAParams()))
	{
		delete params;
		setErrorString(std::string("Error running genetic algorithm: ") + ga->getErrorString());
		delete ga;
		return false;
	}
	delete params;

	if (!ProcessSolution(*ga))
	{
		delete ga;
		return false;
	}
	delete ga;
	
	return true;
}


