/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "cmdsources.h"
#include "graleshellinstance.h"
#include <shellp/fileiosystem.h>
#include <grale/sourceplane.h>
#include <grale/circularsource.h>
#include <grale/ellipticalsource.h>
#include <grale/polygonsource.h>
#include <grale/imagesdata.h>
#include <grale/gravitationallens.h>
#include <grale/discretesource.h>
#include <grale/pointsource.h>
#include <grale/imagesbackprojector.h>
#include <grale/gridfunction.h>

#include "debugnew.h"

using namespace shellp;

CmdSourcePlane::CmdSourcePlane(const std::string &cmdname) : ShellCommand(cmdname)
{
	Dsarg = new ShellCmdRealArg("D_s");
	Dsarg->setUnitType(UNIT_DISTANCE);
	Dsarg->setDescription("Angular diameter distance between source and observer.");
	addArgument(Dsarg);
	Ddsarg = new ShellCmdRealArg("D_ds");
	Ddsarg->setUnitType(UNIT_DISTANCE);
	Ddsarg->setDescription("Angular diameter distance between source and lens.");
	addArgument(Ddsarg);
	delsrcarg = new ShellCmdBoolArg("delcurrentsources","yes");
	delsrcarg->setDescription("Flag indicating if the current sources should be removed or reinstalled in the new source plane.");
	addArgument(delsrcarg);
	setDescription("Create a source plane to which sources can be added and removed.");
}

CmdSourcePlane::~CmdSourcePlane()
{
	delete Ddsarg;
	delete Dsarg;
	delete delsrcarg;
}

bool CmdSourcePlane::execute()
{
	double D_s = Dsarg->getValue();
	double D_ds = Ddsarg->getValue();
	bool delsources = delsrcarg->getValue();
	grale::SourcePlane *srcplane = new grale::SourcePlane(D_s,D_ds);

	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	gsinst->SetSourcePlane(srcplane,delsources);
	
	return true;
}

CmdSourcePlaneLoadSave::CmdSourcePlaneLoadSave(const std::string &cmdname) : ShellCommand(cmdname)
{
	fnamearg = new ShellCmdStringArg("filename");
	addArgument(fnamearg);
	processsourcesarg = new ShellCmdBoolArg("processsources");
	processsourcesarg->setDefault("yes");
	addArgument(processsourcesarg);
}
	
CmdSourcePlaneLoadSave::~CmdSourcePlaneLoadSave()
{
	delete fnamearg;
	delete processsourcesarg;
}

CmdSourcePlaneLoad::CmdSourcePlaneLoad(const std::string &cmdname) : CmdSourcePlaneLoadSave(cmdname)
{
	fnamearg->setDescription("Name of the source plane file you wish to load.");
	processsourcesarg->setDescription("Flag indicating if individual sources should be loaded from the file.");
	setDescription("Load source plane information from a file.");
}

CmdSourcePlaneLoad::~CmdSourcePlaneLoad()
{
}

bool CmdSourcePlaneLoad::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	IOSystem *iosys = gsinst->getIOSystem();

	iosys->writeOutputLine("NOT IMPLEMENTED YET");
	// TODO
	return true;
}

CmdSourcePlaneSave::CmdSourcePlaneSave(const std::string &cmdname) : CmdSourcePlaneLoadSave(cmdname)
{
	fnamearg->setDescription("Name of the file you wish to save the source plane to.");
	processsourcesarg->setDescription("Flag indicating if individual sources should be saved to the file.");
	setDescription("Save source plane information to a file.");
}

CmdSourcePlaneSave::~CmdSourcePlaneSave()
{
}

bool CmdSourcePlaneSave::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	IOSystem *iosys = gsinst->getIOSystem();

	iosys->writeOutputLine("NOT IMPLEMENTED YET");
	// TODO
	return true;
}

CmdSourcesAddBase::CmdSourcesAddBase(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pSrcNumArg = new ShellCmdIntArg("srcnum");
	m_pSrcNumArg->setMin(0);
	m_pSrcNumArg->setMax(65535);
	m_pSrcNumArg->setDescription("A number you wish to associate with the source.");
	addArgument(m_pSrcNumArg);
	m_pXPosArg = new ShellCmdRealArg("x");
	m_pXPosArg->setUnitType(UNIT_ANGULAR);
	m_pXPosArg->setDescription("X component of the source position.");
	addArgument(m_pXPosArg);
	m_pYPosArg = new ShellCmdRealArg("y");
	m_pYPosArg->setUnitType(UNIT_ANGULAR);
	m_pYPosArg->setDescription("Y component of the source position.");
	addArgument(m_pYPosArg);
	m_pBrightnessScale = new ShellCmdRealArg("brightnessscale");
	m_pBrightnessScale->setDefault("1");
	m_pBrightnessScale->setDescription("Scale factor for the brightness of a source.");
	addArgument(m_pBrightnessScale);
}	

CmdSourcesAddBase::~CmdSourcesAddBase()
{
	delete m_pSrcNumArg;
	delete m_pXPosArg;
	delete m_pYPosArg;
	delete m_pBrightnessScale;
}
	
CmdSourcesAddCircular::CmdSourcesAddCircular(const std::string &cmdName) : CmdSourcesAddBase(cmdName)
{
	m_pRadiusArg = new ShellCmdRealArg("radius");
	m_pRadiusArg->setUnitType(UNIT_ANGULAR);
	m_pRadiusArg->setDescription("Radius of the circular source.");
	addArgument(m_pRadiusArg);
	m_pFadeArg = new ShellCmdBoolArg("fade","false");
	m_pFadeArg->setDescription("Flag indicating if the surface brightness of the source should fade near the edge.");
	addArgument(m_pFadeArg);
	setDescription("Add a circular source.");
}

CmdSourcesAddCircular::~CmdSourcesAddCircular()
{
	delete m_pRadiusArg;
	delete m_pFadeArg;
}

bool CmdSourcesAddCircular::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	
	if (pGSInst->GetSource(getSourceNumber()) != 0)
	{
		setErrorString("The specified source number is already taken");
		return false;
	}

	grale::CircularSource *pSrc = new grale::CircularSource(grale::Vector2D<double>(getX(),getY()), m_pRadiusArg->getValue(), getBrightnessScale());
	pSrc->setFade(m_pFadeArg->getValue());

	pGSInst->SetSource(getSourceNumber(), pSrc);
	return true;
}

CmdSourcesAddPoint::CmdSourcesAddPoint(const std::string &cmdName) : CmdSourcesAddBase(cmdName)
{
	setDescription("Add a point source.");
}

CmdSourcesAddPoint::~CmdSourcesAddPoint()
{
}

bool CmdSourcesAddPoint::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	
	if (pGSInst->GetSource(getSourceNumber()) != 0)
	{
		setErrorString("The specified source number is already taken");
		return false;
	}

	grale::PointSource *pSrc = new grale::PointSource(grale::Vector2D<double>(getX(),getY()), getBrightnessScale());

	pGSInst->SetSource(getSourceNumber(), pSrc);
	return true;
}

CmdSourcesAddRotatedBase::CmdSourcesAddRotatedBase(const std::string &cmdName) : CmdSourcesAddBase(cmdName)
{
	m_pAngleArg = new ShellCmdRealArg("angle");
	m_pAngleArg->setDefault("0");
	m_pAngleArg->setDescription("Angle to rotate the source over (in degrees).");
	addArgument(m_pAngleArg);
}

CmdSourcesAddRotatedBase::~CmdSourcesAddRotatedBase()
{
	delete m_pAngleArg;
}

CmdSourcesAddElliptical::CmdSourcesAddElliptical(const std::string &cmdname) : CmdSourcesAddRotatedBase(cmdname)
{
	axisarg = new ShellCmdRealArg("semimajor");
	axisarg->setUnitType(UNIT_ANGULAR);
	axisarg->setDescription("The semimajor axis of the ellipse.");
	addArgument(axisarg);
	excentrarg = new ShellCmdRealArg("eccentricity");
	excentrarg->setMin(0);
	excentrarg->setMax(1);
	excentrarg->setDescription("Eccentricity of the ellipse.");
	addArgument(excentrarg);
	fadearg = new ShellCmdBoolArg("fade","false");
	fadearg->setDescription("Flag indicating if the intensity of the source should fade near the edge.");
	addArgument(fadearg);
	setDescription("Add an elliptical source.");
}

CmdSourcesAddElliptical::~CmdSourcesAddElliptical()
{
	delete axisarg;
	delete excentrarg;
	delete fadearg;
}

bool CmdSourcesAddElliptical::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	
	if (gsinst->GetSource(getSourceNumber()) != 0)
	{
		setErrorString("The specified source number is already taken");
		return false;
	}

	grale::EllipticalSource *src = new grale::EllipticalSource(grale::Vector2D<double>(getX(),getY()), axisarg->getValue(), excentrarg->getValue(), getAngle(), getBrightnessScale());
	src->setFade(fadearg->getValue());

	gsinst->SetSource(getSourceNumber(),src);
	return true;
}

CmdSourcesAddFITS::CmdSourcesAddFITS(const std::string &cmdname) : CmdSourcesAddRotatedBase(cmdname)
{
	fnamearg = new ShellCmdStringArg("filename");
	fnamearg->setDescription("The name of the FITS file.");
	addArgument(fnamearg);
	widtharg = new ShellCmdRealArg("width");
	widtharg->setUnitType(UNIT_ANGULAR);
	widtharg->setDescription("The angular width of the image.");
	addArgument(widtharg);
	heightarg = new ShellCmdRealArg("height");
	heightarg->setUnitType(UNIT_ANGULAR);
	heightarg->setDescription("The angular height of the image.");
	addArgument(heightarg);
	setDescription("Use a FITS image as a source.");
}

CmdSourcesAddFITS::~CmdSourcesAddFITS()
{
	delete fnamearg;
	delete widtharg;
	delete heightarg;
}

bool CmdSourcesAddFITS::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	
	if (gsinst->GetSource(getSourceNumber()) != 0)
	{
		setErrorString("The specified source number is already taken");
		return false;
	}

	grale::DiscreteSource *src = new grale::DiscreteSource(grale::Vector2D<double>(getX(),getY()), getAngle(), getBrightnessScale());

	if (!src->loadFITS(fnamearg->getValue(), widtharg->getValue(), heightarg->getValue()))
	{
		setErrorString(std::string("Couldn't create source from FITS file: ") + src->getErrorString());
		delete src;
		return false;
	}

	gsinst->SetSource(getSourceNumber(),src);
	return true;
}

CmdSourcesAddPolygon::CmdSourcesAddPolygon(const std::string &cmdname) : CmdSourcesAddRotatedBase(cmdname)
{
	fnamearg = new ShellCmdStringArg("filename");
	fnamearg->setDefault("*");
	fnamearg->setDescription("If specified, polygon data is read from this file.");
	addArgument(fnamearg);

	setDescription("Adds a polygon-shaped source.");

	xposarg = new ShellCmdRealArg("x");
	xposarg->setUnitType(UNIT_ANGULAR);
	yposarg = new ShellCmdRealArg("y");
	yposarg->setUnitType(UNIT_ANGULAR);
	pointargs.push_back(xposarg);
	pointargs.push_back(yposarg);
}

CmdSourcesAddPolygon::~CmdSourcesAddPolygon()
{
	delete fnamearg;
	delete xposarg;
	delete yposarg;
}

bool CmdSourcesAddPolygon::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (gsinst->GetSource(getSourceNumber()) != 0)
	{
		setErrorString("The specified source number is already taken");
		return false;
	}

	std::string fname = fnamearg->getValue();
	IOSystem *iosys;
	FileIOSystem fiosys;
	std::string prefix("Polygon point: ");
	
	if (fname == std::string("*")) // use current input
		iosys = gsinst->getIOSystem();
	else
	{
		if (!fiosys.init(fname))
		{
			setErrorString(std::string("Can't open file '") + fname + std::string("' to read pologon data from: ") + fiosys.getErrorString());
			return false;
		}
		iosys = &fiosys;
	}
	
	iosys->writeOutputLine("Enter individual polygon points. Type 'end' to end input.");
	iosys->writeOutputLine("Argument order is: ");
	iosys->writeOutputLine(ShellCmdArg::getArgumentNames(pointargs));
	
	std::list<grale::Vector2D<double> > polygonpoints;
	
	bool done = false;
	bool error = false;
	while (!done)
	{
		std::string line;

		if (!iosys->readInputLine(prefix,line))
			done = true;
		else
		{
			if (iosys->echo())
				iosys->print("%s%s", prefix.c_str(), line.c_str());

			std::vector<std::string> args;

			IOSystem::splitLine(line,args);
			if (args.size() == 1)
			{
				std::string str = args[0];

				if (str == std::string("end"))
					done = true;
			}

			if (!done)
			{
				std::string errstr;
				
				if (!ShellCmdArg::parseArgumentList(pointargs,args,errstr))
				{
					error = true;
					done = true;
					setErrorString(std::string("Error parsing point info: ") + errstr);
				}
				else
				{
					// ok, arguments are valid
					
					double x = xposarg->getValue();
					double y = yposarg->getValue();

					polygonpoints.push_back(grale::Vector2D<double>(x,y));
				}
			}
		}
	}

	if (error)
		return false;

	if (polygonpoints.size() < 3)
	{
		setErrorString("Not enough polygon points specified; a polygon should consist of at least three points");
		return false;
	}
	
	// parameters are filled in, create the source
	
	grale::Polygon2D<double> polygon;

	polygon.init(polygonpoints, false);
	
	grale::PolygonSource *src = new grale::PolygonSource(grale::Vector2D<double>(getX(),getY()), polygon, getAngle(), getBrightnessScale());

	gsinst->SetSource(getSourceNumber(),src);

	return true;
}

CmdSourcesDelete::CmdSourcesDelete(const std::string &cmdname) : ShellCommand(cmdname)
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	
	srcnumarg = new ShellCmdIntArg("srcnum");
	srcnumarg->setMin(0);
	srcnumarg->setMax(65535);
	srcnumarg->setDescription("The number associated with the source.");
	addArgument(srcnumarg);
	setDescription("Remove a specific source from the source plane.");
}

CmdSourcesDelete::~CmdSourcesDelete()
{
	delete srcnumarg;
}

bool CmdSourcesDelete::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	if (!gsinst->GetSource(srcnumarg->getValue()))
	{
		setErrorString("There is no source with associated to the specified number");
		return false;
	}
	gsinst->SetSource(srcnumarg->getValue(),0);
	return true;
}

CmdSourcesClear::CmdSourcesClear(const std::string &cmdname) : ShellCommand(cmdname)
{
	setDescription("Removes all sources.");
}

CmdSourcesClear::~CmdSourcesClear()
{
}

bool CmdSourcesClear::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();

	gsinst->ClearSources();
	return false;
}

CmdSourcesMove::CmdSourcesMove(const std::string &cmdname) : ShellCommand(cmdname)
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	
	srcnumarg = new ShellCmdIntArg("srcnum");
	srcnumarg->setMin(0);
	srcnumarg->setMax(65535);
	srcnumarg->setDescription("The number associated with the source.");
	addArgument(srcnumarg);
	xarg = new ShellCmdRealArg("x");
	xarg->setUnitType(UNIT_ANGULAR);
	xarg->setDescription("X position or offset.");
	addArgument(xarg);
	yarg = new ShellCmdRealArg("y");
	yarg->setUnitType(UNIT_ANGULAR);
	yarg->setDescription("Y position or offset.");
	addArgument(yarg);
	relarg = new ShellCmdBoolArg("relative","false");
	relarg->setDescription("Flag indicating if the specified coordinates are relative to the current coordinates.");
	addArgument(relarg);
	setDescription("Adjust the coordinates of a specific source.");
}

CmdSourcesMove::~CmdSourcesMove()
{
	delete srcnumarg;
	delete xarg;
	delete yarg;
	delete relarg;
}

bool CmdSourcesMove::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::SourceImage *src;

	if ((src = gsinst->GetSource(srcnumarg->getValue())) == 0)
	{
		setErrorString("There is no source with associated to the specified number");
		return false;
	}

	grale::Vector2D<double> newpos(xarg->getValue(),yarg->getValue());
		
	if (relarg->getValue())
		src->addToAngularPosition(newpos);
	else
		src->setAngularPosition(newpos);
	return true;
}

CmdSourcesRotate::CmdSourcesRotate(const std::string &cmdname) : ShellCommand(cmdname)
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	
	srcnumarg = new ShellCmdIntArg("srcnum");
	srcnumarg->setMin(0);
	srcnumarg->setMax(65535);
	srcnumarg->setDescription("The number associated with the source.");
	addArgument(srcnumarg);
	anglearg = new ShellCmdRealArg("angle");
	anglearg->setDescription("Rotation angle (in degrees).");
	addArgument(anglearg);
	relarg = new ShellCmdBoolArg("relative","false");
	relarg->setDescription("Flag indicating if the specified rotation angle is relative to the current angle.");
	addArgument(relarg);
	setDescription("Adjust the rotation angle of a specific source.");
}

CmdSourcesRotate::~CmdSourcesRotate()
{
	delete srcnumarg;
	delete anglearg;
	delete relarg;
}

bool CmdSourcesRotate::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::SourceImage *src;

	if ((src = gsinst->GetSource(srcnumarg->getValue())) == 0)
	{
		setErrorString("There is no source with associated to the specified number");
		return false;
	}

	if (relarg->getValue())
		src->addToAngle(anglearg->getValue());
	else
		src->setAngle(anglearg->getValue());
	return true;
}

CmdSourcePlaneImagesData::CmdSourcePlaneImagesData(const std::string &cmdname) : ShellCommand(cmdname)
{
	numarg = new ShellCmdIntArg("index");
	numarg->setMin(0);
	numarg->setDescription("Index of the images data to select (starting with 0).");
	addArgument(numarg);
	m_pImageNumberArg = new ShellCmdIntArg("imgnum");
	m_pImageNumberArg->setMin(-1);
	m_pImageNumberArg->setDefault("-1");
	m_pImageNumberArg->setDescription("Index of the image to use (from the selected data set). Specify -1 to use all images.");
	addArgument(m_pImageNumberArg);
	m_pUseMagnifications = new ShellCmdBoolArg("demagnify");
	m_pUseMagnifications->setDefault("no");
	m_pUseMagnifications->setDescription("Use the calculated magnification info to demagnify the value of each point in the images data. This "
			                     "should only be done if the data describes point sources.");
	addArgument(m_pUseMagnifications);
	setDescription("Creates a new source plane with one source, by taking the convex hull of the "
		       "back-projected points of one of the images data sets.");
}

CmdSourcePlaneImagesData::~CmdSourcePlaneImagesData()
{
	delete numarg;
	delete m_pImageNumberArg;
	delete m_pUseMagnifications;
}

bool CmdSourcePlaneImagesData::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::GravitationalLens *pLens = gsinst->GetLens();
	std::list<grale::Vector2D<double> > points;
	std::list<grale::ImagesDataExtended *> imgslist;

	gsinst->GetImagesDataList(imgslist);
	if (imgslist.empty())
	{
		setErrorString("No images data instances in the list");
		return false;
	}

	int idx = numarg->getValue();
	if (idx >= imgslist.size())
	{
		char buf[256];

		sprintf(buf,"The highest allowed index value is currently %d", (int)(imgslist.size()-1));
		setErrorString(buf);
		return false;
	}

	std::list<grale::ImagesDataExtended *>::const_iterator it = imgslist.begin();
	for (int i = 0 ; i < idx ; i++)
		it++;

	grale::ImagesDataExtended *img = (*it);

	if (img == 0)
	{
		setErrorString("No images data selected");
		return false;
	}

	if (pLens == 0)
	{
		setErrorString("No lens available");
		return false;
	}
	
	int numImgs = img->getNumberOfImages();

	if (numImgs <= 0)
	{
		setErrorString("Selected images data doesn't contain any images");
		return false;
	}

	int imgNum = m_pImageNumberArg->getValue();

	if (imgNum >= 0 && imgNum >= numImgs)
	{
		char str[256];

		sprintf(str,"%d",numImgs-1);
		setErrorString(std::string("The image index must lie in the range 0..")  + std::string(str));
		return false;
	}

	double D_s = img->getD_s();
	double D_ds = img->getD_ds();

	if (img->hasTriangulation())
	{
		grale::ImagesDataExtended *pImgDat = new grale::ImagesDataExtended(*img);
		
		std::vector<std::list<grale::TriangleIndices> > triangulations(numImgs);

		for (int i = 0 ; i < numImgs ; i++)
		{
			if (imgNum == -1 || imgNum == i)
			{
				if (!pImgDat->getTriangles(i, triangulations[i]))
				{
					setErrorString(std::string("Unable to retrieve triangle data: ") + pImgDat->getErrorString());
					delete pImgDat;
					return false;
				}
			}
		}

		// Project the images back onto the source plane

		std::list<grale::ImagesDataExtended *> imagesDataList;
		
		pImgDat->setD_s(D_s);
		pImgDat->setD_ds(D_ds);

		imagesDataList.push_back(pImgDat);

		grale::ImagesBackProjector bp(*pLens, imagesDataList, 0, false); // z_d doesn't matter here
		
		// Create triangle list and store min and max positions
		
		std::list<grale::Triangle2DPlus<double> > allTriangles;
		double minX = 1e30;
		double maxX = -1e30;
		double minY = 1e30;
		double maxY = -1e30;

		for (int i = 0 ; i < numImgs ; i++)
		{
			const grale::Vector2D<float> *pBetas = bp.getBetas(0, i); // only one source
			const float *pIntensities = bp.getOriginalIntensities(0, i);
			const float *pInvMag = bp.getInverseMagnifications(0, i);
			std::list<grale::TriangleIndices>::const_iterator it, endIt;

			it = triangulations[i].begin();
			endIt = triangulations[i].end();
			
			while (it != endIt)
			{
				grale::TriangleIndices indices = (*it);
				grale::Vector2D<float> fp1 = pBetas[indices.getIndex(0)];
				grale::Vector2D<float> fp2 = pBetas[indices.getIndex(1)];
				grale::Vector2D<float> fp3 = pBetas[indices.getIndex(2)];
				grale::Vector2D<double> p1 = grale::Vector2D<double>(fp1.getX(), fp1.getY()) * bp.getAngularScale();
				grale::Vector2D<double> p2 = grale::Vector2D<double>(fp2.getX(), fp2.getY()) * bp.getAngularScale();
				grale::Vector2D<double> p3 = grale::Vector2D<double>(fp3.getX(), fp3.getY()) * bp.getAngularScale();
				double h1 = (double)pIntensities[indices.getIndex(0)] * bp.getIntensityScale();
				double h2 = (double)pIntensities[indices.getIndex(1)] * bp.getIntensityScale();
				double h3 = (double)pIntensities[indices.getIndex(2)] * bp.getIntensityScale();
				
				if (m_pUseMagnifications->getValue())
				{
					h1 *= (double)ABS(pInvMag[indices.getIndex(0)]);
					h2 *= (double)ABS(pInvMag[indices.getIndex(1)]);
					h3 *= (double)ABS(pInvMag[indices.getIndex(2)]);
				}
				
				minX = MIN(minX, p1.getX());
				minX = MIN(minX, p2.getX());
				minX = MIN(minX, p3.getX());
				minY = MIN(minY, p1.getY());
				minY = MIN(minY, p2.getY());
				minY = MIN(minY, p3.getY());
				maxX = MAX(maxX, p1.getX());
				maxX = MAX(maxX, p2.getX());
				maxX = MAX(maxX, p3.getX());
				maxY = MAX(maxY, p1.getY());
				maxY = MAX(maxY, p2.getY());
				maxY = MAX(maxY, p3.getY());

				allTriangles.push_back(grale::Triangle2DPlus<double>(p1, p2, p3, h1, h2, h3));

				it++;
			}
		}
		
		delete pImgDat;

		// Create gridfunction and plot it

		grale::GridFunction gridFunction(allTriangles, grale::Vector2D<double>(minX, minY), grale::Vector2D<double>(maxX, maxY), 128, 128, false, true, 4);
	
		grale::DiscreteSource *pSrc = new grale::DiscreteSource(grale::Vector2D<double>(0,0), 0, 1.0);

		if (!pSrc->setGridFunction(&gridFunction))
		{
			setErrorString(std::string("Unable to initialize the discrete source: ") + pSrc->getErrorString());
			delete pSrc;
			return false;
		}
		
		grale::SourcePlane *pSrcPlane = new grale::SourcePlane(D_s, D_ds);
		gsinst->SetSourcePlane(pSrcPlane, true);
			
		gsinst->SetSource(0, pSrc);
	}
	else
	{	
		int numimages = img->getNumberOfImages();
		for (int i = 0 ; i < numimages ; i++)
		{
			if (imgNum == -1 || imgNum == i)
			{
				int numpoints = img->getNumberOfImagePoints(i);
				for (int p = 0 ; p < numpoints ; p++)
				{
					grale::Vector2D<double> beta;
					grale::Vector2D<double> theta = img->getImagePointPosition(i, p);

					if (!pLens->traceTheta(D_s, D_ds, theta, &beta))
					{
						setErrorString(std::string("Error projecting point: ") + pLens->getErrorString());
						return false;
					}
					
					points.push_back(beta);
				}
			}
		}

		grale::Polygon2D<double> polygon;

		polygon.init(points, true);

		const grale::Vector2D<double> *points2 = polygon.getPoints();
		int num = polygon.getNumberOfPoints();
		grale::Vector2D<double> center(0,0);

		for (int i = 0 ; i < num ; i++)
			center += points2[i];
		center /= (double)num;

		points.clear();
		for (int i = 0 ; i < num ; i++)
		{
			grale::Vector2D<double> diff = points2[i]-center;
			points.push_back(diff);
		}

		grale::Polygon2D<double> polygon2;

		polygon2.init(points,false);
		
		grale::SourcePlane *srcplane = new grale::SourcePlane(D_s,D_ds);
		gsinst->SetSourcePlane(srcplane,true);

		grale::PolygonSource *src = new grale::PolygonSource(center, polygon2, 0, 1.0);
		gsinst->SetSource(0, src);
	}	
	return true;
}

CmdSourcePlaneImagesDataPointSource::CmdSourcePlaneImagesDataPointSource(const std::string &cmdname) : ShellCommand(cmdname)
{
	numarg = new ShellCmdIntArg("index");
	numarg->setMin(0);
	numarg->setDescription("Index of the images data to select (starting with 0).");
	addArgument(numarg);
	m_pImageNumberArg = new ShellCmdIntArg("imgnum");
	m_pImageNumberArg->setMin(-1);
	m_pImageNumberArg->setDefault("-1");
	m_pImageNumberArg->setDescription("Index of the image to use (from the selected data set). Specify -1 to use all images.");
	addArgument(m_pImageNumberArg);
	m_pRadiusArg = new ShellCmdRealArg("radius");
	m_pRadiusArg->setUnitType(UNIT_ANGULAR);
	m_pRadiusArg->setDefault("-1");
	m_pRadiusArg->setDescription("If positive, a circular source with this radius will be used instead of a point image.");

	addArgument(m_pRadiusArg);
	setDescription("Creates a new source plane with one point source, by taking the average position "
		       "of the back-projected points of (one of) the images data sets. If the radius is "
		       "positive, instead of a point source a circular source with this radius will be used.");
}

CmdSourcePlaneImagesDataPointSource::~CmdSourcePlaneImagesDataPointSource()
{
	delete numarg;
	delete m_pImageNumberArg;
	delete m_pRadiusArg;
}

bool CmdSourcePlaneImagesDataPointSource::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::GravitationalLens *pLens = gsinst->GetLens();
	std::list<grale::Vector2D<double> > points;
	std::list<grale::ImagesDataExtended *> imgslist;

	gsinst->GetImagesDataList(imgslist);
	if (imgslist.empty())
	{
		setErrorString("No images data instances in the list");
		return false;
	}

	int idx = numarg->getValue();
	if (idx >= imgslist.size())
	{
		char buf[256];

		sprintf(buf,"The highest allowed index value is currently %d", (int)(imgslist.size()-1));
		setErrorString(buf);
		return false;
	}

	std::list<grale::ImagesDataExtended *>::const_iterator it = imgslist.begin();
	for (int i = 0 ; i < idx ; i++)
		it++;

	grale::ImagesDataExtended *img = (*it);

	if (img == 0)
	{
		setErrorString("No images data selected");
		return false;
	}

	if (pLens == 0)
	{
		setErrorString("No lens available");
		return false;
	}
	
	int numImgs = img->getNumberOfImages();

	if (numImgs <= 0)
	{
		setErrorString("Selected images data doesn't contain any images");
		return false;
	}

	int imgNum = m_pImageNumberArg->getValue();

	if (imgNum >= 0 && imgNum >= numImgs)
	{
		char str[256];

		sprintf(str,"%d",numImgs-1);
		setErrorString(std::string("The image index must lie in the range 0..")  + std::string(str));
		return false;
	}

	double D_s = img->getD_s();
	double D_ds = img->getD_ds();

	int numImages = img->getNumberOfImages();
	grale::Vector2D<double> avgBeta;
	int count = 0;

	for (int i = 0 ; i < numImages ; i++)
	{
		int numPoints = img->getNumberOfImagePoints(i);

		for (int p = 0 ; p < numPoints ; p++)
		{
			//std::cout << i << " " << p << std::endl;
			grale::Vector2D<double> theta = img->getImagePointPosition(i, p);
			grale::Vector2D<double> beta;

			//      TODO: error check
			pLens->traceTheta(D_s, D_ds, theta, &beta);

			avgBeta += beta;
			count++;	
		}
	}

	// TODO: zero check
	avgBeta /= (double)count;

	grale::SourcePlane *srcplane = new grale::SourcePlane(D_s, D_ds);
	gsinst->SetSourcePlane(srcplane,true);

	double radius = m_pRadiusArg->getValue();

	if (radius < 0)
	{
		grale::PointSource *src = new grale::PointSource(avgBeta, 1.0);
		gsinst->SetSource(0, src);
	}
	else
	{
		grale::CircularSource *pSrc = new grale::CircularSource(avgBeta, radius, 1);
		gsinst->SetSource(0, pSrc);
	}

	return true;
}


